﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace REG_MARK_LIB
{
    public class RegMark
    {
        //массив кодов региона, которые не находятся в диапазоне от 1 до 99
        static List<int> codeRegion = new List<int> { 102, 103, 111, 113, 116, 118, 121, 122, 123, 124, 125, 126, 130, 134, 136, 138, 142, 147, 150, 152, 154, 155, 156, 158, 159, 161, 163, 164, 172, 173, 174, 177, 178, 180, 181, 186, 190, 193, 196, 197, 198, 199, 252, 277, 299, 702, 716, 725, 750, 754, 761, 763, 774, 777, 790, 797, 799, 977 };
        //массив символов которые могут быть в номере
        static List<char> charMark = new List<char> { 'a', 'b', 'e', 'k', 'm', 'h', 'o', 'p', 'c', 't', 'y', 'x' };

        /// <summary>
        /// Проверяет переданный номерной знак в формате a999aa999
        /// </summary>
        /// <param name="mark">Номерной знак в формате a999aa999</param>
        /// <returns>Возвращает правильность номерного знака</returns>
        public static bool CheckMark(string mark)
        {
            mark = mark.ToLower();//меняем регистр на нижний
            Regex checkMark = new Regex("^[abekmhopctyx][0-9]{3}[abekmhopctyx]{2}[0-9]{2,3}$"); //регулярное выражение для проверки номера на корректность записи
            bool flag = true; //состояние корректности номера

            //проверка на корректность записи
            if (!checkMark.IsMatch(mark))
            {
                return false;//выходим потому что номер некорректный
            }

            //проверка кода региона 
            string code = mark.Substring(6);
            if (code.Length == 2)
            {
                //если code входит в диапазон от 1 до 99 - верно, иначе - ложь
                flag = (int.Parse(code) >= 1 && int.Parse(code) <= 99) ? true : false;
            }
            else
            {
                //если codeRegion содержит в себе code - верно, иначе - ложь
                flag = (codeRegion.Contains(int.Parse(code))) ? true : false;
            }

            return flag;
        }

        /// <summary>
        /// Создание номера, следующего после текущего в данной серии или создание номера следующей серии
        /// </summary>
        /// <param name="mark">Номерной знак в формате a999aa999</param>
        /// <returns>Возвращает новый номер</returns>
        public static string GetNextMarkAfter(string mark)
        {
            //проверяем номер на существование, если не существует возвращаем NULL
            if (!CheckMark(mark)) throw new Exception("Номера не существует");
            mark = mark.ToLower();//меняем регистр на нижний             
            int num = int.Parse(mark.Substring(1, 3));//номер машины
            //проверяем номер, если меньше 999 - создаем номер той же серии
            if (num < 999)
            {
                string n = (num + 1).ToString();
                while(n.Length != 3){
                    n = "0" + n;
                }
                return mark[0] + n + mark.Substring(4);//возвращаем новый номер
            }
            //все серийные буквы номера
            char ch1 = mark[0];
            char ch2 = mark[4];
            char ch3 = mark[5];
            if (ch1 != 'x')//если 1 буква не х
            {
                int i = charMark.IndexOf(ch1);//индекс следующей буквы
                return charMark[i + 1] + "001" + mark.Substring(4);//возвращаем новый номер
            }
            else if (ch3 != 'x')//если 3 буква не х
            {
                int i = charMark.IndexOf(ch3);//индекс следующей буквы
                return mark[0] + "001" + mark[4] + charMark[i + 1] + mark.Substring(5);//возвращаем новый номер
            }
            else if (ch2 != 'x')//если 2 буква не х
            {
                int i = charMark.IndexOf(ch2);//индекс следующей буквы
                return mark[0] +"001" + charMark[i + 1] + mark[5] + mark.Substring(6);//возвращаем новый номер
            }

            //если вдруг все буквы были ххх
            throw new Exception("Номер крайний");

        }

        /// <summary>
        /// Выдает следующий, после переданного номера prevMark, номер в промежутке между rangeStart и rangeEnd включительно
        /// </summary>
        /// <param name="prevMark">Номерной знак в формате a999aa999</param>
        /// <param name="rangeStart">Начало промежутка номеров</param>
        /// <param name="rangeEnd">Конец промежутка номера</param>
        /// <returns></returns>
        public static string GetNextMarkAfterInRange(string prevMark, string rangeStart, string rangeEnd)
        {
            //проверяем номера на существование, если не существует возвращаем NULL
            if (!CheckMark(prevMark) || !CheckMark(rangeStart) || !CheckMark(rangeEnd)) return null;

            //меняем регистр на нижний
            prevMark = prevMark.ToLower();
            rangeStart = rangeStart.ToLower();
            rangeEnd = rangeEnd.ToLower();

            //проверяем на корректность промежутка
            if (!compareMarks(rangeStart, rangeEnd)) throw new Exception("Промежуток номеров построен некорректно");
            //проверяем номер на выход из-за границы
            if (!compareMarks(prevMark, rangeEnd)) return "out of stock";

            //проверяем номер на вхождение в промежуток
            if (compareMarks(rangeStart, prevMark))
            {
                string newMark = GetNextMarkAfter(prevMark);//создаем новый номер
                if (compareMarks(newMark, rangeEnd)) return newMark;
                else throw new Exception("Номер выходит за заданный промежуток");
            }
            else
            {
                return rangeStart;//возвращаем начало промежутка, так как он является 1 числом
            }
        }
        /// <summary>
        /// Метод проверки существования промежутка
        /// </summary>
        /// <param name="rangeStart">Начало</param>
        /// <param name="rangeEnd">Конец</param>
        /// <returns></returns>
        /// <exception cref="Exception"></exception>
        private static bool compareMarks(string rangeStart, string rangeEnd)
        {
            if (rangeStart.Substring(6) != rangeEnd.Substring(6)) throw new Exception("У номеров разный код регионов");
            //все серийные буквы номера
            int n1s = charMark.IndexOf(rangeStart[0]);
            int n2s = charMark.IndexOf(rangeStart[4]);
            int n3s = charMark.IndexOf(rangeStart[5]);
            int n1e = charMark.IndexOf(rangeEnd[0]);
            int n2e = charMark.IndexOf(rangeEnd[4]);
            int n3e = charMark.IndexOf(rangeEnd[5]);

            //сравнение 2 символов
            if (n2s > n2e)
            {
                return false;
            }
            //сравнение 3 символов
            if (n3s > n3e)
            {
                return false;
            }
            //сравнение 1 символов
            if (n1s > n1e)
            {
                return false;
            }
            //сравнение номеров при условии что символы совпадают
            int nums = int.Parse(rangeStart.Substring(1, 3));//номер машины
            int nume = int.Parse(rangeEnd.Substring(1, 3));//номер машины
            if (n2s == n2e && n3s == n3e && n1s == n1e && nums > nume)
            {
                return false;
            }

            return true;
        }
        /// <summary>
        /// Поиск количества возможных номеров между переданными номерами
        /// </summary>
        /// <param name="mark1">Номерной знак в формате a999aa999</param>
        /// <param name="mark2">Номерной знак в формате a999aa999</param>
        /// <returns>Количество номеров</returns>
        public static int GetCombinationsCountInRange(string mark1, string mark2)
        {
            //проверяем номера на существование, если не существует возвращаем NULL
            if (!CheckMark(mark1) || !CheckMark(mark2)) throw new Exception("номера не корректны");

            //меняем регистр на нижний 
            mark1 = mark1.ToLower();
            mark2 = mark2.ToLower();

            //проверяем на корректность промежутка
            if (!compareMarks(mark1, mark2)) throw new Exception("Промежуток номеров построен некорректно");

            //смотрим количество цифр до конца диапазона
            int nums = int.Parse(mark1.Substring(1, 3));//номер машины
            int nume = int.Parse(mark2.Substring(1, 3));//номер машины
            //все серийные буквы номера
            int n1s = charMark.IndexOf(mark1[0]);
            int n2s = charMark.IndexOf(mark1[4]);
            int n3s = charMark.IndexOf(mark1[5]);
            int n1e = charMark.IndexOf(mark2[0]);
            int n2e = charMark.IndexOf(mark2[4]);
            int n3e = charMark.IndexOf(mark2[5]);

            if (n1s == n1e && n2s == n2e && n3e == n3s)//если у номеров одинаковые буквы
            {
                int c = 0;//количество номеров
                for (int i = nums; i <= nume; i++)
                {
                    c++;
                }
                return c;
            }

            int count = 0;//количество номеров
            for (int u = n2s; u <= n2e; u++)
            {
                int r2 = (u == n2s) ? n2s : 0;//начало номера
                int r2End = (u == n2e) ? n2e : charMark.Count - 1;//конец номера

                for (int z = r2; z <= r2End; z++)
                {
                    int r3 = (z == n3s) ? n1s : 0;//начало номера
                    int r3End = (z == n3e) ? n1e : charMark.Count - 1;//конец номера

                    for (int i = r3; i <= r3End; i++)
                    {
                        int r1 = (i == n1s) ? nums : 0;//начало номера
                        int r1End = (i == n1e) ? nume : 999;//конец номера

                        for (int j = r1; j <= r1End; j++)
                        {
                            count++;
                        }
                    }
                }
            }
            return count;
        }
    }
}
